package App::ArchLinux::PackagerTools;
use v5.24;
use strictures;

our $VERSION = "0.01";

use autodie;
use Function::Parameters;
use Log::Any qw($log);

use App::ArchLinux::PackagerTools::Pacman;
use App::ArchLinux::PackagerTools::CPAN;
use App::ArchLinux::PackagerTools::Config;

=encoding utf-8

=head1 NAME

App::ArchLinux::PackagerTools - It's new $module

=head1 SYNOPSIS

    use App::ArchLinux::PackagerTools;

=head1 DESCRIPTION

App::ArchLinux::PackagerTools is ...

=head1 METHODS

=head2 Constructors

=head3 new

 App::ArchLinux::PackagerTools->new();

Returns a new instance.

=cut
method new($class: $context = {}, $deps = {}) {
	$deps->{config} //= App::ArchLinux::PackagerTools::Config->new($context);
	$deps->{cpan} //= App::ArchLinux::PackagerTools::CPAN->new($context);
	$deps->{pacman} //= App::ArchLinux::PackagerTools::Pacman->new($context);
	return $class->new_no_defaults($context, $deps);
}

method new_no_defaults($class: $context, $deps = {}) {
	my $self = {};
	bless $self, $class;
	$self->{deps} = $deps;
	$self->{config} = $deps->{config}->get_config();
	return $self;
}

=head3 get_distributions_in_repo

 my $dists = $app->get_distributions_in_repo();

Return an arrayref of hashrefs with CPAN distribution information. See
L<App::ArchLinux::PackagerTools::Pacman/"get_perl_distributions">.

=cut

method get_distributions_in_repo() {
	return $self->{deps}->{pacman}->get_perl_distributions();
}

=head3 get_updateable_packages

 my $distribution_names = ['App::ArchLinux::PackagerTools', 'DBI'];
 my $packages = $app->get_updateable_packages($distribution_names);
 print $packages->[0]->{dist_name};
 print $packages->[0]->{pkgname};
 print $packages->[0]->{repo_version};
 print $packages->[0]->{cpan_version};

Accepts a list of package names and returns a list of packages that can be
updated. The returned packages contain the name of the distribution, the name
of the pacman package, and the versions of each, the pacman package (which is
older) and the CPAN distribution.

=cut

method get_updateable_packages($distribution_names) {
	my @packages;
	my $repo_versions = $self->{deps}->{pacman}->get_perl_distribution_versions($distribution_names);
	for my $dist_name ($distribution_names->@*) {
		#my $repo_version = $self->{deps}->{pacman}->get_perl_distribution_version($dist_name);
		my $repo_version = $repo_versions->{$dist_name};
		my $dist = $self->{deps}->{cpan}->get_dist($dist_name);
		if ($self->{deps}->{cpan}->is_newer_version_available($dist_name, $repo_version)) {
			my $pkgname = $self->{deps}->{pacman}->get_perl_pkgname($dist_name);
			$log->infof('New version available for dist \'%s\' in package \'%s\': %s -> %s', $dist_name, $pkgname, $repo_version, $dist->{version});
			push @packages, {
				dist_name => $dist_name,
				repo_version => $repo_version,
				cpan_version => $dist->{version},
				pkgname => $pkgname,
			};
		} else {
			$log->debugf('Dist \'%s\' is already up to date with version %s', $dist_name, $dist->{version});
		}

	}
	return \@packages;
}

=head3 get_maintainer

 my $maintainer = $app->get_maintainer($pkgname);

Accepts a package name and returns the maintainer of the package.

=cut

method get_maintainer($pkgname) {
	# TODO fetch maintainer from archweb + cache the value. use CHI
	return "bluewind\@xinu.at";
}

=head3 add_maintainers

 my $packages = $app->get_updateable_packages($distribution_names);
 my $packages_with_maintainer = $app->add_maintainers($packages);
 print $packages_with_maintainer->[0]->{maintainer};

Accepts a list of packages (hashes with the pkgname key) and returns a list
with the maintainer key added to each package.

=cut

method add_maintainers($packages) {
	my @packages_with_maintainer;
	# fetch package maintainer from archweb
	for my $package ($packages->@*) {
		push @packages_with_maintainer, {
			$package->%*,
			maintainer => $self->get_maintainer($package->{pkgname}),
		};
	}
	return \@packages_with_maintainer;
}

=head1 LICENSE

Copyright (C) 2018  Florian Pritz E<lt>bluewind@xinu.atE<gt>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


=head1 AUTHOR

Florian Pritz E<lt>bluewind@xinu.atE<gt>

=cut

1;

__END__
